-----------------------------------------------------------------------------------------------
-- Client Lua Script for CostumeQuickMenu
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "CostumesLib"
require "ChatSystemLib"
require "GameLib"
 
-----------------------------------------------------------------------------------------------
-- CostumeQuickMenu Module Definition
-----------------------------------------------------------------------------------------------
local CostumeQuickMenu = {} 
local CostumeNames
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 local function strsplit(sep, str)
		local sep, fields = sep or ":", {}
		local pattern = string.format("([^%s]+)", sep)
		string.gsub(str ,pattern, function(c) fields[#fields+1] = c end)
		return fields
end
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function CostumeQuickMenu:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
	self.tItems = {}
    return o
end

function CostumeQuickMenu:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		"Character",
		"Costumes",
		"CostumeNames",
		"ChatLog",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 
function CostumeQuickMenu:OnDependencyError(strDep, strError)
	-- if you don't care about this dependency, return true.
	-- if you return false, or don't define this function
	-- any Addons/Packages that list you as a dependency
	if strDep == "CostumeNames" then
		return true
	end
	return false
end
-----------------------------------------------------------------------------------------------
-- CostumeQuickMenu OnLoad
-----------------------------------------------------------------------------------------------
function CostumeQuickMenu:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("CostumeQuickMenu.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- CostumeQuickMenu OnDocLoaded
-----------------------------------------------------------------------------------------------
function CostumeQuickMenu:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "CostumeQuickMenuForm", nil, self)
		self.wndEmote = Apollo.LoadForm(self.xmlDoc, "EmoteCostumeChange", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)
		self.wndEmote:Show(false, true)
		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded",	"OnInterfaceMenuLoaded", self)
		Apollo.RegisterEventHandler("GenericEvent_ToggleCostumeQuickMenu",	"OnToggleQuickMenu", self)
		Apollo.RegisterEventHandler("PlayerLevelChange", "OnLevelUp", self)
		self.tmrCooldown = ApolloTimer.Create(15.0, false, "OnCooldownEnded", self)
		self.tmrCooldown:Stop()
		-- Do additional Addon initialization here
		CostumeNames = Apollo.GetAddon("CostumeNames")
		self:PopulateItemList()
	end
end

function CostumeQuickMenu:OnInterfaceMenuLoaded()
	local tData = {"GenericEvent_ToggleCostumeQuickMenu", "",""}
	Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "Costume Quick Menu" , tData)
	local iMaj, iMin, iPatch = unpack(strsplit(".", XmlDoc.CreateFromFile("toc.xml"):ToTable().Version))
	Event_FireGenericEvent("OneVersion_ReportAddonInfo", "Costume Quick Menu", tonumber(iMaj), tonumber(iMin), tonumber(iPatch))
end

function CostumeQuickMenu:OnToggleQuickMenu()
	local arLoc = Apollo.GetMouse()
	local nXoff = self.wndMain:GetWidth()
	local nYoff = self.wndMain:GetHeight()
	self.wndMain:SetAnchorOffsets(arLoc.x, arLoc.y - nYoff, arLoc.x + nXoff, arLoc.y)
	self.wndMain:Show(true, false)
	self:PopulateItemList()
end

function CostumeQuickMenu:OnLevelUp()
	self:PopulateItemList()
end

-----------------------------------------------------------------------------------------------
-- CostumeQuickMenu Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- populate item list
function CostumeQuickMenu:PopulateItemList()
	-- make sure the item list is empty to start with
	local wndGrid = self.wndMain:FindChild("wndGrid")
	
	wndGrid:DeleteAll()
	
	local nCostumeTotal = CostumesLib.GetCostumeCount()
	for i = 1,nCostumeTotal do
	    if CostumeNames ~= nil then
			wndGrid:AddRow(CostumeNames:GetName(i))
		else
			wndGrid:AddRow("Costume  " .. i)
		end
	end
end

function CostumeQuickMenu:OnCooldownEnded()
	self.wndMain:FindChild("wndGrid"):Enable(true)
	self.wndMain:FindChild("wndGrid"):SetOpacity(1)
end
-----------------------------------------------------------------------------------------------
-- CostumeQuickMenuForm Functions
---------------------------------------------------------------------------------------------

--
function CostumeQuickMenu:OnToggleSlotVisible( wndHandler, wndControl, eMouseButton )
	if Apollo.IsControlKeyDown() then
		self.wndMain:Show(false, true)
		self.wndEmote:Show(true, true)
		self.wndEmote:SetData({type = "Toggle", data = wndControl:GetName()})
	else
		local cstCostume = CostumesLib.GetCostume(CostumesLib.GetCostumeIndex())
		local nSlotID = GameLib.CodeEnumItemSlots[wndControl:GetName()]
		cstCostume:SetSlotVisible(nSlotID  , not cstCostume:IsSlotVisible(nSlotID))
		cstCostume:SaveChanges()
	end
end

function CostumeQuickMenu:OnSelectionChanged( wndHandler, wndControl, iRow, iCol)
	if Apollo.IsControlKeyDown() then
		self.wndMain:Show(false, true)
		self.wndEmote:Show(true, true)
		self.wndEmote:SetData({type = "Emote", data = iRow})
	else
		CostumesLib.SetCostumeIndex(iRow)
		wndControl:Enable(false)
		wndControl:SetOpacity(0.5)
		self.tmrCooldown:Start()
		self.wndMain:Show(false, true)
	end
end

-----------------------------------------------------------------------------------------------
-- EmoteCostumeChange Functions
---------------------------------------------------------------------------------------------------

function CostumeQuickMenu:OnEmoteChangeOK( wndHandler, wndControl, eMouseButton )
	local ebEmote = self.wndEmote:FindChild("ebEmote")
	local strEmote = ebEmote:GetText()
	local tInfo = self.wndEmote:GetData()
	
	local uPlayer = GameLib.GetPlayerUnit()
	-- do our action
	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_Emote, strEmote, uPlayer:GetName(), uPlayer)
	
	if tInfo.type == "Emote" then
		CostumesLib.SetCostumeIndex(tInfo.data)
	elseif tInfo.type == "Toggle" then
		local cstCostume = CostumesLib.GetCostume(CostumesLib.GetCostumeIndex())
		local nSlotID = GameLib.CodeEnumItemSlots[tInfo.data]
		cstCostume:SetSlotVisible(nSlotID  , not cstCostume:IsSlotVisible(nSlotID))
		cstCostume:SaveChanges()
	end
	
	-- cleanup for next time.
	self.wndMain:FindChild("wndGrid"):Enable(false)
	self.wndMain:FindChild("wndGrid"):SetOpacity(0.5)
	self.tmrCooldown:Start()
	self.wndEmote:SetData(nil)
	ebEmote:SetText("")
	self.wndEmote:Show(false, true)
end

function CostumeQuickMenu:OnEmoteChangeCancel( wndHandler, wndControl, eMouseButton )
	local ebEmote = self.wndEmote:FindChild("ebEmote")
	self.wndEmote:SetData(nil)
	ebEmote:SetText("")
	self.wndEmote:Show(false, true)
end

-----------------------------------------------------------------------------------------------
-- CostumeQuickMenu Instance
-----------------------------------------------------------------------------------------------
local CostumeQuickMenuInst = CostumeQuickMenu:new()
CostumeQuickMenuInst:Init()
